/* -*- c++ -*- */
/*
 * Copyright 2004,2013 Free Software Foundation, Inc.
 *
 * This file is part of GNU Radio
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 */

#ifndef INCLUDED_GR_ENDIANNESS_H
#define INCLUDED_GR_ENDIANNESS_H

namespace gr {

typedef enum { GR_MSB_FIRST, GR_LSB_FIRST } endianness_t;

} /* namespace gr */

#endif /* INCLUDED_GR_ENDIANNESS_H */
